/*
 ************************************************************************************************
 *                                                                                              *
 *  OSA cooperative RTOS for Microchip PIC-controllers: PIC10/12/16/18/24/dsPIC                 *
 *                                                                                              *
 *  URL:            http://wiki.pic24.ru/doku.php/en/osa/ref/intro                              *
 *                  http://picosa.narod.ru                                                      *
 *                                                                                              *
 *----------------------------------------------------------------------------------------------*
 *                                                                                              *
 *  File:           osa_ccs16.h                                                                 *
 *                                                                                              *
 *  Compilers:      CCS                                                                         *
 *                                                                                              *
 *  Programmer:     Timofeev Victor                                                             *
 *                  osa@pic24.ru, testerplus@mail.ru                                            *
 *                                                                                              *
 *  Description:    CCS16 specific definitions                                                  *
 *                                                                                              *
 *  History:        26.06.2009                                                                  *
 *                                                                                              *
 ************************************************************************************************
 */


#ifndef __OSACCS16_H__
#define __OSACCS16_H__



/************************************************************************************************
 *                                                                                              *
 *     Integer types definitions                                                                * 
 *                                                                                              *
 ************************************************************************************************/

#define __osa_uint8     unsigned int8
#define __osa_uint16    unsigned int16
#define __osa_uint32    unsigned int32
#define __osa_bool      unsigned int8

#define _OST_INT_TYPE   __osa_uint8
#define _OST_INT_MASK    7
#define _OST_INT_SHIFT   3
#define _OST_INT_SIZE    8



/************************************************************************************************
 *                                                                                              * 
 *     Location keywords                                                                        * 
 *                                                                                              * 
 ************************************************************************************************/

#define NEAR
#define bank0
#define bank1
#define bank2
#define bank3

#define bit             _OST_INT_TYPE
#define __osa30_near

//#define _OS_CUR_FLAGS_IN_OS_state       // Work with tasks state througth OS_state variable
#define _OS_CUR_FLAGS_IN_FSR       // Work with tasks state througth OS_state variable



/************************************************************************************************
 *                                                                                              * 
 *     Processor specified definitions for system flags                                         * 
 *     (see OST_SYSTEM_FLAGS and OST_TCB definition in file "OSA.h")                            * 
 *                                                                                              * 
 ************************************************************************************************/

#define _OST_SYSTEM_FLAGS_PROC_SPEC                                                             \
                                                                                                \
        /*5*/   _OST_INT_TYPE   bTimeout    : 1; /* Event waiting timeout               */      \
        /*6*/   _OST_INT_TYPE   bGIE_CTemp  : 1; /* Saved GIE while in critical section */      \

#define _OST_TCB_PROC_SPEC      // Empty macro for this processor


/************************************************************************************************
 *                                                                                              *
 *     Registers definitions                                                                    *
 *                                                                                              *
 ************************************************************************************************/

#byte   _fsr        = 0x04
#byte   _status     = 0x03
#byte   _indf       = 0x00
#byte   _pcl        = 0x02
#byte   _pclath     = 0x0A
#byte   _intcon     = 0x0B

#bit    _carry      = _status.0
#bit    _zero       = _status.2
#bit    _irp        = _status.7
#bit    _gie        = _intcon.7


extern __osa_uint16    _OS_RetAddr_Temp;

/************************************************************************************************
 *                                                                                              *
 *     Constants and types                                                                      *
 *                                                                                              *
 ************************************************************************************************/

#ifndef OS_SMSG_SIZE
#define OS_SMSG_SIZE    0
#endif

//------------------------------------------------------------------------------
#define _FSR_TYPE               __osa_uint8

#if sizeof(char*) == 1
    #define OS_RAM_POINTER_SIZE     1
#endif

#if sizeof(char*) == 2
    #define OS_RAM_POINTER_SIZE     2
#endif

//------------------------------------------------------------------------------

#define OS_CODE_POINTER_SIZE        2
#define OST_CODE_POINTER            __osa_uint16



/************************************************************************************************
 *                                                                                              *
 *     Platform specific macros                                                                 * 
 *                                                                                              *
 ************************************************************************************************/


#define     OS_CLRWDT()         #asm clrwdt #endasm
#define     OS_SLEEP()          #asm sleep  #endasm


//------------------------------------------------------------------------------
#ifdef _OS_CUR_FLAGS_IN_FSR
//------------------------------------------------------------------------------

    //------------------------------------------------------------------------------
    #define OS_CUR_FLAGS_DEFINED
    //------------------------------------------------------------------------------
    
    #define _OS_bTaskReady              ((NEAR OST_TASK_STATE)_indf).bReady
    #define _OS_bTaskCanContinue        ((NEAR OST_TASK_STATE)_indf).bCanContinue
    #define _OS_bTaskDelay              ((NEAR OST_TASK_STATE)_indf).bDelay
    #define _OS_bTaskTimeout          (!((NEAR OST_TASK_STATE)_indf).bDelay)
    #define _OS_bTaskEnable             ((NEAR OST_TASK_STATE)_indf).bEnable
    #define _OS_cTaskPriority           ((NEAR OST_TASK_STATE)_indf).cPriority
    #define _OS_bTaskPaused             ((NEAR OST_TASK_STATE)_indf).bPaused

//------------------------------------------------------------------------------
#endif      // _OS_CUR_FLAGS_IN_FSR
//------------------------------------------------------------------------------




/************************************************************************************************
 *                                                                                              *
 *     Context switching macros                                                                 *
 *                                                                                              *
 ************************************************************************************************/


extern #separate void __OS_JumpToTask (void);


#define _OS_JumpToTask()    __OS_JumpToTask()


#define _OS_SaveRetAddr()                           \
    #asm                                            \
        clrf    _status                             \
    #endasm                                         \

    
#define _OS_if_Ready()  _OS_SET_FSR_CUR_TASK();     \
                        if (_OS_CheckIndfReady())   


 
//------------------------------------------------------------------------------

extern #separate void OS_SchedRetPoint (void);
extern #separate void OS_SchedRetPointNoSave (void);
extern #separate void OS_ClearReady (void);
extern #separate void OS_ClearReadySetCanContinue (void);

    
//------------------------------------------------------------------------------
#define _OS_RETURN_SAVE()                               \
    {                                                   \
        _OS_RetAddr_Temp = label_address(label);        \
        OS_SchedRetPoint(0);                            \
        #asm return #endasm                             \
    label:;                                             \
        #undef label                                    \
    }
//------------------------------------------------------------------------------
#define _OS_CLEAR_READY()                               \
    {                                                   \
        _OS_RetAddr_Temp = label_address(label);        \
        OS_SchedRetPoint(1);                            \
        #asm return #endasm                             \
    label:;                                             \
        #undef label                                    \
    }
//------------------------------------------------------------------------------
#define _OS_CLEAR_READY_SET_CANCONTINUE()               \
    {                                                   \
        _OS_RetAddr_Temp = label_address(label);        \
        OS_SchedRetPoint(2);                            \
        #asm return #endasm                             \
    label:;                                             \
        #undef label                                    \
    }
//------------------------------------------------------------------------------
#define _OS_RETURN_NO_SAVE()                            \
        #asm return #endasm                             \

//------------------------------------------------------------------------------

//void __OS_SET_FSR_CUR_TASK(char w);
void __OS_SET_FSR_CUR_TASK (void);

#define _OS_SET_FSR_CUR_TASK()      __OS_SET_FSR_CUR_TASK()
#define _OS_SET_FSR_CUR_TASK_W(w)   {__OS_SET_FSR_CUR_TASK();_fsr+=w;}

 











//#define _OS_DTimersWork_DEFINED
//
//    #define __OS_DTimersWork()                                              \
//        {                                                                   \
//            _OS_DTimerWork = (OST_DTIMER*)&_OS_DTimers;                     \
//            while (_OS_DTimerWork->Flags.bNextEnable)                       \
//            {                                                               \
//              _OS_DTimerWork = (OST_DTIMER*)_OS_DTimerWork->Next;           \
//              _OS_DTimerWork->Timer++;                                      \
//              if (!(_OS_DTimerWork->Timer))                                 \
//                _OS_DTimerWork->Flags.bTimeout = 1;                         \
//            }                                                               \
//        }


/******************************************************************************************
 *
 *  WORK WITH DYNAMIC TIMERS in OS_Timer
 *
 ******************************************************************************************/

//------------------------------------------------------------------------------
#ifdef OS_ENABLE_DTIMERS
//------------------------------------------------------------------------------

#define _OS_DTimersWork_DEFINED

#define _OS_IndfTimerTimeout       ((NEAR OST_DTIMER_FLAGS)(_indf)).bTimeout
#define _OS_IndfTimerActive        ((NEAR OST_DTIMER_FLAGS)(_indf)).bActive
#define _OS_IndfTimerRun           ((NEAR OST_DTIMER_FLAGS)(_indf)).bRun
#define _OS_IndfTimerNextEnable    ((NEAR OST_DTIMER_FLAGS)(_indf)).bNextEnable



#if OS_RAM_POINTER_SIZE == 1
    #define _OS_SER_FSR_DTIMER_NEXT()     _fsr = _indf
    #define _OS_SET_FSR_DTIMER_TIMER()    
#endif

#if OS_RAM_POINTER_SIZE == 2

    #define _OS_SER_FSR_DTIMER_NEXT()                       \
            #asm                                            \
                movf    _indf, w                            \
                incf    _fsr, f                             \
                /* here status,z can't be = '1'   */        \
                btfsc   _indf, 0                            \
                bsf     _status, 2                          \
                movwf   _fsr                                \
                bcf     _status, 7                          \
                btfsc   _status, 2                          \
                bsf     _status, 7                          \
            #endasm                                         \

    #define _OS_SET_FSR_DTIMER_TIMER()    _fsr++

#endif



//------------------------------------------------------------------------------
#if   OS_DTIMER_SIZE == 1
//------------------------------------------------------------------------------

    #define OS_INC_DTIMER()                                 \
        {                                                   \
            /* Now W = fsr */                               \
            _OS_SET_FSR_DTIMER_TIMER();                     \
            #asm                                            \
                incf    _fsr, f                             \
                incf    _fsr, f                             \
                bcf     _status, 2                          \
                incf    _indf, f                            \
                movwf   _fsr                                \
            #endasm                                         \
            if (_zero) _OS_IndfTimerTimeout = 1;            \
        }

//------------------------------------------------------------------------------
#elif OS_DTIMER_SIZE == 2
//------------------------------------------------------------------------------

    #define OS_INC_DTIMER()                                 \
        {                                                   \
            /* Now W = fsr */                               \
            _OS_SET_FSR_DTIMER_TIMER();                     \
            #asm                                            \
                bcf     _status, 0                          \
                incf    _fsr, f                             \
                incf    _fsr, f                             \
                incfsz  _indf, f                            \
                goto    label                               \
                incf    _fsr, f                             \
                incfsz  _indf, f                            \
                goto    label                               \
                bsf     _status, 0                          \
        label:                                              \
                movwf   _fsr                                \
            #endasm                                         \
            if (_carry) _OS_IndfTimerTimeout = 1;           \
        }

//------------------------------------------------------------------------------
#elif OS_DTIMER_SIZE == 4
//------------------------------------------------------------------------------

    #define OS_INC_DTIMER()                                 \
        {                                                   \
            /* Now W = fsr */                               \
            _OS_SET_FSR_DTIMER_TIMER();                     \
            #asm                                            \
                bcf     _status, 0                          \
                incf    _fsr, f                             \
                incf    _fsr, f                             \
                incfsz  _indf, f                            \
                goto    label                               \
                incf    _fsr, f                             \
                incfsz  _indf, f                            \
                goto    label                               \
                incf    _fsr, f                             \
                incfsz  _indf, f                            \
                goto    label                               \
                incf    _fsr, f                             \
                incfsz  _indf, f                            \
                goto    label                               \
                bsf     _status, 0                          \
            label:                                          \
                movwf   _fsr                                \
            #endasm                                         \
            if (_carry) _OS_IndfTimerTimeout = 1;           \
        }

//------------------------------------------------------------------------------
#endif  // OS_DTIMER_SIZE
//------------------------------------------------------------------------------



#define __OS_DTimersWork()                                  \
    {                                                       \
        _irp = 0;                                           \
        _fsr = (_FSR_TYPE) &_OS_DTimers;                    \
    REPEAT:                                                 \
        if (_OS_IndfTimerNextEnable)                        \
        {                                                   \
            _fsr++;                                         \
            _OS_SER_FSR_DTIMER_NEXT();                      \
            if (!_OS_IndfTimerRun) goto REPEAT;             \
            OS_INC_DTIMER();                                \
            goto REPEAT;                                    \
        }                                                   \
    }


//------------------------------------------------------------------------------
#endif  // OS_ENABLE_DTIMER
//------------------------------------------------------------------------------







 





/************************************************************************************************
 *                                                                                              * 
 *     Create, replace, reserve task specific services                                          * 
 *                                                                                              * 
 ************************************************************************************************/


/************************************************************************/
/*                                                                      */
/* Replace current task with new (current task will be deleted)         */
/*                                                                      */
/************************************************************************/

#define OS_Task_Replace(priority, TaskName)                                         \
    {                                                                               \
        _OS_SET_FSR_CUR_TASK();                                                     \
        _indf = priority | 0x48; /* bEnable  bReady */                             \
        _OS_CurTask->pTaskPointer = ((OST_CODE_POINTER)TaskName);                   \
        _OS_Return();                                                               \
    }


/************************************************************************/
/*                                                                      */
/* Create task                                                          */
/*                                                                      */
/************************************************************************/

#define OS_Task_Create(priority, TaskName)                          \
    {                                                               \
        _OS_Task_Create(priority, (OST_CODE_POINTER)TaskName);      \
     }

//        OS_Task_Reserve(TaskName);                                  \

/************************************************************************/
/*                                                                      */
/* Create task by indirrect address                                     */
/*                                                                      */
/************************************************************************/

#define OS_Task_Reserve(TaskName)                                   \
        if (_pclath & 0x80) TaskName();                             \




/************************************************************************************************
 *                                                                                              *
 *                                     I N T E R R U P T S                                      * 
 *                                                                                              *
 ************************************************************************************************/


extern  char OS_DI (void);
#define OS_RI(cSavedInt)    {if (cSavedInt & 0x80) _gie = 1;}
#define OS_EI()             { _gie = 1;}



/************************************************************************************************
 *                                                                                              *
 *                                     C O N D I T I O N S                                      *
 *                                                                                              *
 ************************************************************************************************/


#define _OS_CHECK_EVENT(event) if (!_OS_CheckEvent(event)) _OS_Return()












#endif
